use strict;
use XML::XPath;
use XML::XPath::XMLParser;

#these two items could go into the ini file
#set to 1 to create generic items from unknown types
my $createunknown =1;
#set to 1 to create groups even when they have the same name as items by suffixing _group 
my $smartgroupfix =1;

#set up the item types
#I missed the compool(?), as I couldn't see an example anywhere
my %itemtype;
$itemtype{"X10I"} = "X10_Item";
$itemtype{"X10A"} = "X10_Appliance";
$itemtype{"X10O"} = "X10_Ote";
$itemtype{"X10SL"} = "X10_Switchlinc";
$itemtype{"X10G"} = "X10_Garage_Door";
$itemtype{"X10MS"} = "X10_Sensor";
$itemtype{"RF"} = "RF_Item";
$itemtype{"IBUTTON"} = "iButton";
$itemtype{"SERIAL"} = "Serial";
$itemtype{"STARGATEDIN"} = "StargateDigitalInput";
$itemtype{"STARGATEVAR"} = "StargateVariable";
$itemtype{"STARGATEFLAG"} = "StargateFlag";
$itemtype{"STARGATERELAY"} = "StargateRelay";
$itemtype{"STARGATETHERM"} = "StargateThermostat";
$itemtype{"STARGATEPHONE"} = "StargateTelephone";
$itemtype{"SG485LCD"} = "StargateLCDKeypad";
$itemtype{"SG485RCSTHRM"} = "StargateRCSThermostat";
$itemtype{"XANTECH"} = "Xantech_Zone";
$itemtype{"MP3PLAYER"} = "Mp3Player";
$itemtype{"AUDIOTRON"} = "AudiotronPlayer";
$itemtype{"WEATHER"} = "Weather_Item";
$itemtype{"X10S"} = "X10_IrrigationController";
$itemtype{"X10T"} = "RCS_Item";
$itemtype{"GENERIC"} = "Generic_Item";


my (%groups, %objects);


sub read_table_init_xml_A {
  # reset known groups
	print_log "Initialized read_table_xml.pl";
	%groups=();
	%objects=();
}


sub read_table_xml_A {
my $data = $_[0];                                            #perl complains about trying to fiddle directly with @_[0]
if (!$data) {print "No data in sub read_table_xml_A";next;}
#do some mying
my ($code,$name,%nodeset,$object);
my $count = 1;
my $itemsearch = "/items/item[$count]/*";
                                        
$data =~s/\s*(<items>.*<\/items>)\s*/$1/;                    #some basic cleaning
my ($record) = XML::XPath->new("xml", $data);                #load xml
    
 while (my $items = $record->find($itemsearch))  {
  foreach my $node ($items->get_nodelist) {                  #break out all the individual nodes in each item
   $name = $node->getName();
   if ($nodeset{$name}){
    $nodeset{$name} .= ",'".$node->string_value()."'";
   } else {
    $nodeset{$name} = "'".$node->string_value()."'";         #easier to quote everything now even though it is probably slower
   }
	}
	
#unquote those that don't need it
	
  $nodeset{"type"} =~ s/'//g;                              
  $nodeset{"name"} =~ s/'//g;
	
#do all the requires
  
	if(uc($nodeset {"type"}) =~/STARGATE/) {                
   require 'StarGate.pm';
  }

  if(uc($nodeset {"type"}) =~/SG485/) {
   require 'Stargate485.pm';
  }

  if(uc($nodeset {"type"}) =~/XANTECH/) {
   require 'Xantech.pm';
  }

  if(uc($nodeset {"type"}) =~/X10T/) {
   require 'RCS_Item.pm';
  }

#set up all objects
	
  if(uc($nodeset {"type"}) =~/^X10(I|A|O|G|MS|SL)|RF|IBUTTON|SERIAL|STARGATE(IN|VAR|FLAG|RELAY|THERM|PHONE)|SG485(LCD|RCSTHRM)|XANTECH/) {
      $object = $itemtype{uc($nodeset {"type"})}."(".$nodeset {"address"}.",".$nodeset {"other"}.")";
  }

  elsif(uc($nodeset {"type"}) =~/^MP3PLAYER'/) {
  require 'Mp3Player.pm';
  $object = $itemtype{uc($nodeset {"type"})}."(".$nodeset {"address"}.")";
  }

  elsif(uc($nodeset {"type"}) =~/^AUDIOTRON/) {
  require 'AudiotronPlayer.pm';
  $object = $itemtype{uc($nodeset {"type"})}."(".$nodeset {"address"}.")";
  }
  
	elsif(uc($nodeset {"type"}) =~/^WEATHER|X10(S|T)/) {    
  $object = $itemtype{uc($nodeset {"type"})}."(".$nodeset {"address"}.")";
  }

  elsif(uc($nodeset {"type"}) =~/^GENERIC/) {
  $object = $itemtype{uc($nodeset {"type"})};
  }

  elsif(uc($nodeset {"type"}) =~/^VOICE/) {
  my $vcommand = $nodeset {"other"}; 
  my $fixedname = $nodeset {"name"};
  $fixedname =~ s/_/ /g;
   if (!($vcommand =~ /.*\[.*/)) {
   $vcommand .= " [ON,OFF]";
   }
  $code .= sprintf "\nmy \$v_%s_state;\n", $nodeset {"name"};
  $code .= sprintf "\$v_%s = new Voice_Cmd(\"%s\");\n", $nodeset {"name"}, $vcommand;
  $code .= sprintf "if (\$v_%s_state = said \$v_%s) {\n", $nodeset {"name"}, $nodeset {"name"};
  $code .= sprintf "  set \$%s \$v_%s_state;\n", $nodeset {"name"}, $nodeset {"name"};
  $code .= sprintf "  respond \"Turning %s \$v_%s_state\";\n", $fixedname, $nodeset {"name"};
  $code .= sprintf "}\n";
	}
 else {
  if ($createunknown == 1){
  print "\nUnrecognized .mht entry: $record\nCreating as generic object\n";
  $object = $itemtype{"GENERIC"};
  } else {
  print "\nUnrecognized .mht entry: $record\n";
  }
 }
 
#if object exists then create some code for it
 if ($object) {
 my $code2 = sprintf "\n\$%-35s =  new %s;\n", $nodeset{"name"}, $object;
 $code2 =~ s/= *new \S+ *\(/-> add \(/ if $objects{$nodeset{"name"}}++;
 $code .= $code2;
 }

#if groups exist create them and add items
 if ($nodeset{"group"}) { 
  for my $group (split('\',\'',$nodeset{"group"})) {
  $group =~ s/\'//g;
	my $continue = 1; #I'm sure there is a perl command to exit the for loop, but I can't remember it right now
   if ($nodeset{"name"} eq $group) {
	  if ($smartgroupfix == 1) {
		$group .= "_group";
		} else {
    print_log "mht object and group name are the same: $name  Bad idea!";
		$continue = 0;
    
    }
	 }
   if ($continue == 1) {
   $code .= sprintf "\$%-35s =  new Group;\n", $group unless $groups{$group};
   $code .= sprintf "\$%-35s -> add(\$%s);\n", $group, $nodeset{"name"} unless $groups{$group}{$nodeset {"name"}};
   $groups{$group}{$name}++;
   }

   if(lc($group) eq 'hidden'){
   $code .= sprintf "\$%-35s -> hidden(1);\n", $nodeset {"name"};
   }
  }
 }
 #set/reset everything that needs resetting
 $count++;
 $itemsearch = "/items/item[$count]/*"; 
 %nodeset = ();
} 


return $code;

    
}

1;
